/*
 * sfuntmpl_basic.c: Basic 'C' template for a level 2 S-function.
 *
 *  -------------------------------------------------------------------------
 *  | See matlabroot/simulink/src/sfuntmpl_doc.c for a more detailed template |
 *  -------------------------------------------------------------------------
 *
 * Copyright 1990-2002 The MathWorks, Inc.
 * $Revision: 1.27 $
 */

 
 
 
 
 

/*
 * You must specify the S_FUNCTION_NAME as the name of your S-function
 * (i.e. replace sfuntmpl_basic with the name of your S-function).
 */

#define S_FUNCTION_NAME  Init_VRatio_ChgRss_b
#define S_FUNCTION_LEVEL 2
#define D_pi             3.1415926

#define D_min(a,b) (((a)<(b))?(a):(b))
#define D_max(a,b) (((a)>(b))?(a):(b))

/*
 * Need to include simstruc.h for the definition of the SimStruct and
 * its associated macro definitions.
 */
#include "simstruc.h"



/* Error handling
 * --------------
 *
 * You should use the following technique to report errors encountered within
 * an S-function:
 *
 *       ssSetErrorStatus(S,"Error encountered due to ...");
 *       return;
 *
 * Note that the 2nd argument to ssSetErrorStatus must be persistent memory.
 * It cannot be a local variable. For example the following will cause
 * unpredictable errors:
 *
 *      mdlOutputs()
 *      {
 *         char msg[256];         {ILLEGAL: to fix use "static char msg[256];"}
 *         sprintf(msg,"Error due to %s", string);
 *         ssSetErrorStatus(S,msg);
 *         return;
 *      }
 *
 * See matlabroot/simulink/src/sfuntmpl_doc.c for more details.
 */

/*====================*
 * S-function methods *
 *====================*/

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    /* See sfuntmpl_doc.c for more details on the macros below */

    ssSetNumSFcnParams(S,3);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        /* Return if number of expected != number of actual parameters */
        return;
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 1)) return;
    ssSetInputPortWidth(S, 0, 1);
    ssSetInputPortRequiredContiguous(S, 0, true); /*direct input signal access*/
    /*
     * Set direct feedthrough flag (1=yes, 0=no).
     * A port has direct feedthrough if the input is used in either
     * the mdlOutputs or mdlGetTimeOfNextVarHit functions.
     * See matlabroot/simulink/src/sfuntmpl_directfeed.txt.
     */
    ssSetInputPortDirectFeedThrough(S, 0, 1);

    if (!ssSetNumOutputPorts(S, 5)) return;
    ssSetOutputPortWidth(S, 0, 1);
    ssSetOutputPortWidth(S, 1, 1);
    ssSetOutputPortWidth(S, 2, 1);
    ssSetOutputPortWidth(S, 3, 128);
    ssSetOutputPortWidth(S, 4, 98304);
   

    ssSetNumSampleTimes(S,1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetOptions(S, 0);
}

  
/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *    This function is used to specify the sample time(s) for your
 *    S-function. You must register the same number of sample times as
 *    specified in ssSetNumSampleTimes.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);

}



#define MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#if defined(MDL_INITIALIZE_CONDITIONS)
  /* Function: mdlInitializeConditions ========================================
   * Abstract:
   *    In this function, you should initialize the continuous and discrete
   *    states for your S-function block.  The initial states are placed
   *    in the state vector, ssGetContStates(S) or ssGetRealDiscStates(S).
   *    You can also perform any other initialization activities that your
   *    S-function may require. Note, this routine will be called at the
   *    start of simulation and if it is present in an enabled subsystem
   *    configured to reset states, it will be call when the enabled subsystem
   *    restarts execution to reset the states.
   */
  static void mdlInitializeConditions(SimStruct *S)
  {
  }
#endif /* MDL_INITIALIZE_CONDITIONS */



#define MDL_START  /* Change to #undef to remove function */
#if defined(MDL_START) 
  /* Function: mdlStart =======================================================
   * Abstract:
   *    This function is called once at start of model execution. If you
   *    have states that should be initialized once, this is the place
   *    to do it.
   */
  static void mdlStart(SimStruct *S)
  {
    const real_T *Vdc = (const real_T*) ssGetInputPortSignal(S,0);
    
    
    const real_T       *V1Ratio = (const real_T*)mxGetPr(ssGetSFcnParam(S,0));
    const real_T       *V2Ratio = (const real_T*)mxGetPr(ssGetSFcnParam(S,1));
    const real_T       *V3Ratio = (const real_T*)mxGetPr(ssGetSFcnParam(S,2));
    
    real_T       *NLevel = ssGetOutputPortSignal(S,0);
    real_T       *Vc1_c = ssGetOutputPortSignal(S,1);
    real_T       *Vc2_c = ssGetOutputPortSignal(S,2);
    real_T       *RSSTable = ssGetOutputPortSignal(S,3);
    real_T       *RSSTable1 = ssGetOutputPortSignal(S,4);
    
    const int        T3[8] = {0,0,0,0,1,1,1,1};
    const int        T2[8] = {0,0,1,1,0,0,1,1};
    const int        T1[8] = {0,1,0,1,0,1,0,1};

    int   Vg_c[8];
    int   V1,V2,V3;
    
    int   SortIndex[24][2],Sort[8][10];
    int   N,SignV1,SignV2,I1D,I2D,ID,ID_o;
    int   FV,FdeltVPN,Fis,FdeltVP,FV1,FV2,FV_o;
	int   Priority,Best,NRss,NRss1,SubPriority,SubBest;
    int   i,j,k,g,k1;
	int	  Index,Row;
    int   ph1,ph2,ph3,sa,sb,sc,smax,smin,sa_c,sb_c,sc_c,sai,sbi,sci;
	
    
    
    V1 = V1Ratio[0];
    V2 = V2Ratio[0];
    V3 = V3Ratio[0];
    
    Vg_c[0] = 0;
    Vg_c[1] = V1Ratio[0];
    Vg_c[2] = V2Ratio[0]-V1Ratio[0];
    Vg_c[3] = V2Ratio[0];
    Vg_c[4] = V3Ratio[0]-V2Ratio[0];
    Vg_c[5] = V3Ratio[0]-V2Ratio[0]+V1Ratio[0];
    Vg_c[6] = V3Ratio[0]-V1Ratio[0];
    Vg_c[7] = V3Ratio[0];
    
    for(i=0;i<24;i++)
    {   SortIndex[i][0] = 0;
        SortIndex[i][1] = 0;
    }
	for(i=0;i<8;i++)
		for(j=0;j<10;j++)
			Sort[i][j] = 0;
    
    for(i=0;i<8;i++)
        SortIndex[Vg_c[i]][0] = 1;
    
    Index = 0;    
    for(i=0;i<24;i++)
    {
        if(SortIndex[i][0]== 1)
            SortIndex[i][1] = Index++;
    }
    N = Index;
    
    for(i=0;i<8;i++)
    {   
        Index = SortIndex[Vg_c[i]][1];
        Sort[Index][0] = Vg_c[i];                   //state Level
        Sort[Index][1] ++;                          //number of redundancy states
        Sort[Index][Sort[Index][1]+1] = i;
    }
     

    for(ph1=0;ph1<N;ph1++)
		for(ph2=0;ph2<N;ph2++)
			for(ph3=0;ph3<N;ph3++)
				for(FdeltVPN=0;FdeltVPN<=5;FdeltVPN++)
					for(FV=0;FV<=1;FV++)
						for(Fis=0;Fis<=1;Fis++)
							for(FV_o=0;FV_o<=1;FV_o++)
						{
                            sa = ph1;
                            sb = ph2;
                            sc = ph3;
                            
                            smax = D_max(sa,sb);
                            smax = D_max(smax,sc);
                            
                            smin = D_min(sa,sb);
                            smin = D_min(smin,sc);
                            
                            NRss = N - (smax - smin);
                            
                            sai = sa - smin;
                            sbi = sb - smin;
                            sci = sc - smin;
    
                            Best = 0;   
                            SubBest = 0;

                            for(k=0;k<NRss;k++)
                            {
                                if((FdeltVPN==0)||(FdeltVPN==1))
									Index = SortIndex[sai][1];
								else if((FdeltVPN==2)||(FdeltVPN==3))
									Index = SortIndex[sbi][1];
								else if((FdeltVPN==4)||(FdeltVPN==5))
									Index = SortIndex[sci][1];
								
								NRss1 = Sort[Index][1];
								
								for(k1=0;k1<NRss1;k1++)
								{   	
										Priority = 1;
										SubPriority = 1;

										Row = Sort[Index][2+k1];
										SignV1 = T1[Row]-T2[Row];
										SignV2 = T2[Row]-T3[Row];
                                    
										if(Fis == 0)			//Is <0
        				    			{
    				        				I1D = SignV1;
        				    				I2D = SignV2;
    				        			}
        				    			else if(Fis == 1)		//Is >0
    				        			{	
        				    				I1D = -SignV1;
    				        				I2D = -SignV2;
        				    			}

										if((FdeltVPN==0)||(FdeltVPN==2)||(FdeltVPN==4))
										{	ID = I1D;
											ID_o = I2D;
										}
										else if((FdeltVPN==1)||(FdeltVPN==3)||(FdeltVPN==5))
										{	ID = I2D;
											ID_o = I1D;
										}
										
										if(((FV==1)&&(ID==-1))||
                						   ((FV==0)&&(ID==1)))
												Priority +=6;
										if(ID==0)
												Priority +=3;
										
										if(((FV_o==1)&&(ID_o==-1))||
                						   ((FV_o==0)&&(ID_o==1)))
												SubPriority +=6;
										if(ID_o==0)
												SubPriority +=3;
												
										if(Priority>Best)
										{
											Best = Priority;
                                            SubBest = SubPriority;
											sa_c = sai;
											sb_c = sbi;
											sc_c = sci;
											RSSTable1[ph1*192*N*N+ph2*192*N+ph3*192+FdeltVPN*32+FV*16+Fis*8+FV_o*4+0] = sa_c;
											RSSTable1[ph1*192*N*N+ph2*192*N+ph3*192+FdeltVPN*32+FV*16+Fis*8+FV_o*4+1] = sb_c;
											RSSTable1[ph1*192*N*N+ph2*192*N+ph3*192+FdeltVPN*32+FV*16+Fis*8+FV_o*4+2] = sc_c;
											RSSTable1[ph1*192*N*N+ph2*192*N+ph3*192+FdeltVPN*32+FV*16+Fis*8+FV_o*4+3] = Row;
                                    
										}
										else if(Priority==Best)
										{	
											if(SubPriority > SubBest)	
											{
												Best = Priority;

												sa_c = sai;
												sb_c = sbi;
												sc_c = sci;
												RSSTable1[ph1*192*N*N+ph2*192*N+ph3*192+FdeltVPN*32+FV*16+Fis*8+FV_o*4+0] = sa_c;
												RSSTable1[ph1*192*N*N+ph2*192*N+ph3*192+FdeltVPN*32+FV*16+Fis*8+FV_o*4+1] = sb_c;
												RSSTable1[ph1*192*N*N+ph2*192*N+ph3*192+FdeltVPN*32+FV*16+Fis*8+FV_o*4+2] = sc_c;
												RSSTable1[ph1*192*N*N+ph2*192*N+ph3*192+FdeltVPN*32+FV*16+Fis*8+FV_o*4+3] = Row;
                                    
											}

										}
                                }//k1
                                sai = sai+1;
                                sbi = sbi+1;
                                sci = sci+1;
                            }// for k
						}//for


    for(i=0;i<N;i++)
    {
		for(FdeltVP=0;FdeltVP<=1;FdeltVP++)
        {
            for(Fis=0;Fis<=1;Fis++)
            {
				for(FV1=0;FV1<=1;FV1++)        
                {
                    for(FV2=0;FV2<=1;FV2++)
                    {
                        Index  = SortIndex[i][1];
                        NRss = Sort[Index][1];
                        Best = 0;
                        for(k=0;k<NRss;k++)
                        {   
                            Priority = 1;
                    
                            Row = Sort[Index][2+k];
                            SignV1 = T1[Row]-T2[Row];
                            SignV2 = T2[Row]-T3[Row];
                            
    						if(Fis == 0)			//Is <0
				    		{
    							I1D = SignV1;
				    			I2D = SignV2;
    						}
				    		else if(Fis == 1)		//Is >0
    						{	
				    			I1D = -SignV1;
    							I2D = -SignV2;
				    		}
                            if(FdeltVP ==0)         //V1 has higher priority, DeltV1Percentage>DeltV2Percentage
    						{
        						if(((FV1==1)&&(I1D==-1))||
        						    ((FV1==0)&&(I1D==1)))
                                    Priority +=6;
                                if(I1D==0)
                                    Priority +=3;
                                if(((FV2==1)&&(I2D==-1))||
        						    ((FV2==0)&&(I2D==1)))
                                    Priority +=2;
                                if(I2D==0)
                                    Priority +=1;
    						}
    						
                            if(FdeltVP ==1)         //V2 has higher priority, DeltV2Percentage>DeltV1Percentage
    						{
        						if(((FV2==1)&&(I2D==-1))||
        						    ((FV2==0)&&(I2D==1)))
                                    Priority +=6;
                                if(I2D==0)
                                    Priority +=3;
                                if(((FV1==1)&&(I1D==-1))||
        						    ((FV1==0)&&(I1D==1)))
                                    Priority +=2;
                                if(I1D==0)
                                    Priority +=1;
    						}
                            
                            if(Priority>=Best)
                            {
                                Best = Priority;
                                RSSTable[i*16+FV1*8+FV2*4+Fis*2+FdeltVP] = Row;
                            }
                       }     
                    }
                }
            }
        }
    }
    NLevel[0] = N;
    Vc1_c[0] = (Vdc[0]*V1)/V3;
    Vc2_c[0] = (Vdc[0]*V2)/V3;
    
   
   
  }
#endif /*  MDL_START */



/* Function: mdlOutputs =======================================================
 * Abstract:
 *    In this function, you compute the outputs of your S-function
 *    block. Generally outputs are placed in the output vector, ssGetY(S).
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    
}


#define MDL_UPDATE  /* Change to #undef to remove function */
#if defined(MDL_UPDATE)
  /* Function: mdlUpdate ======================================================
   * Abstract:
   *    This function is called once for every major integration time step.
   *    Discrete states are typically updated here, but this function is useful
   *    for performing any tasks that should only take place once per
   *    integration step.
   */
  static void mdlUpdate(SimStruct *S, int_T tid)
  {
  }
#endif /* MDL_UPDATE */



#define MDL_DERIVATIVES  /* Change to #undef to remove function */
#if defined(MDL_DERIVATIVES)
  /* Function: mdlDerivatives =================================================
   * Abstract:
   *    In this function, you compute the S-function block's derivatives.
   *    The derivatives are placed in the derivative vector, ssGetdX(S).
   */
  static void mdlDerivatives(SimStruct *S)
  {
  }
#endif /* MDL_DERIVATIVES */



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{
}


/*======================================================*
 * See sfuntmpl_doc.c for the optional S-function methods *
 *======================================================*/

/*=============================*
 * Required S-function trailer *
 *=============================*/

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
