function [sys,x0,str,ts] = PWMControl_bM(t,x,u,flag,JustVal,fsw,F3ndH)
%PWMControl in M-file

%
% Dispatch the flag. The switch function controls the calls to 
% S-function routines at each simulation stage of the S-function.
%
switch flag,
  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  % Initialize the states, sample times, and state ordering strings.
  case 0
    [sys,x0,str,ts]=mdlInitializeSizes(fsw);

  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  % Return the outputs of the S-function block.
  case 3
    sys=mdlOutputs(t,x,u,JustVal,fsw,F3ndH);

  %%%%%%%%%%%%%%%%%%%
  % Unhandled flags %
  %%%%%%%%%%%%%%%%%%%
  % There are no termination tasks (flag=9) to be handled.
  % Also, there are no continuous or discrete states,
  % so flags 1,2, and 4 are not used, so return an emptyu
  % matrix 
  case { 1, 2, 4, 9 }
    sys=[];

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Unexpected flags (error handling)%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Return an error message for unhandled flag values.
  otherwise
    error(['Unhandled flag = ',num2str(flag)]);

end

% end timestwo

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(fsw)

sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 18;  % dynamically sized
sizes.NumInputs      = 98449;  % dynamically sized
sizes.DirFeedthrough = 1;   % has direct feedthrough
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
str = [];
x0  = [];
ts  = [1/fsw 0];   % inherited sample time

% end mdlInitializeSizes

%
%=============================================================================
% mdlOutputs
% Return the output vector for the S-function
%=============================================================================
%
function sys = mdlOutputs(t,x,u,JustVal,fsw,F3ndH)
%Extract Inputs
    k = 1;
    time = u(k);k=k+1;
    Theta_c = u(k);k=k+1;
    NLevel = u(k);k=k+1;
    mbar = u(k);k=k+1;
    FV1	= u(k:k+2);k=k+3;
    FV2	= u(k:k+2);k=k+3;
    Fis = u(k:k+2);k=k+3;
    FdeltVP = u(k:k+2);k=k+3;
    FdeltVPN = u(k);k=k+1;
    RSSTable = u(k:k+127);k=k+128;
    RSSTable1 = u(k:k+98303);k=k+98304;

%Define constants
    T3table = [0,0,0,0,1,1,1,1]';
	T2table = [0,0,1,1,0,0,1,1]';
	T1table = [0,1,0,1,0,1,0,1]';
    
    mind = 0.00001;
    maxd = 1-mind;


    if (mbar>1.14)
        mu = 1.14;
    elseif(mbar<0)
        mu = 0;
    else
        mu = mbar;
    end
    
    Tpos = time;
    
    Tsw = 1/fsw;

    
    if(F3ndH == 0)       %without 3nd harmonic injection
        m = mu;
        Da = (1+m*cos(Theta_c))/2;
        Db = (1+m*cos(Theta_c-2*pi/3))/2;
        Dc = (1+m*cos(Theta_c+2*pi/3))/2;
    else
        m = mu;
        Da = (1+m*cos(Theta_c)-m*cos(3*Theta_c)/6)/2;
        Db = (1+m*cos(Theta_c-2*pi/3)-m*cos(3*Theta_c)/6)/2;
        Dc = (1+m*cos(Theta_c+2*pi/3)-m*cos(3*Theta_c)/6)/2;
    end
    
    if(Da > maxd)
        Da = maxd;
    elseif(Da < mind)
        Da = mind;
    end 
    if(Db > maxd)
        Db = maxd;
    elseif(Db < mind)
        Db = mind;
    end
    if(Dc > maxd)
        Dc = maxd;
    elseif(Dc < mind)
        Dc = mind;    
    end
    
    Dam = (NLevel-1)*Da;
    Dbm = (NLevel-1)*Db;
    Dcm = (NLevel-1)*Dc;
    
    la = floor(Dam); 
    lb = floor(Dbm);
    lc = floor(Dcm);
    
    ta = (Dam-la)*Tsw;    
    tb = (Dbm-lb)*Tsw;
    tc = (Dcm-lc)*Tsw;
    
    Sal = la;
    Sah = la + 1;
    Sbl = lb;
    Sbh = lb + 1;
    Scl = lc;
    Sch = lc + 1;
    
    lowtime = 5.0e-6;
    hightime = Tsw - lowtime;
    
    if(ta > hightime)
        ta = hightime;
    elseif(ta < lowtime)
        ta = lowtime;
    end
    if(tb > hightime)
        tb = hightime;
    elseif(tb < lowtime)
        tb = lowtime;
    end
    if(tc > hightime)
        tc = hightime;
    elseif(tc < lowtime)
        tc = lowtime;
    end
    
% calculate for  window 1234      
         
    Sa_c(1) = Sah;
    Sb_c(1) = Sbh;
    Sc_c(1) = Sch;
        
    Sa_c(4) = Sal;
    Sb_c(4) = Sbl;
    Sc_c(4) = Scl;
    
    if((ta<tb)&&(ta<tc))
        Sa_c(2) = Sal;
        Sb_c(2) = Sbh;
        Sc_c(2) = Sch;
    elseif((tb<ta)&&(tb<tc))
        Sa_c(2) = Sah;
        Sb_c(2) = Sbl;
        Sc_c(2) = Sch;
    else
        Sa_c(2) = Sah;
        Sb_c(2) = Sbh;
        Sc_c(2) = Scl;
    end
    
    if((ta>tb)&&(ta>tc))
        Sa_c(3) = Sah;
        Sb_c(3) = Sbl;
        Sc_c(3) = Scl;
    elseif((tb>ta)&&(tb>tc))
        Sa_c(3) = Sal;
        Sb_c(3) = Sbh;
        Sc_c(3) = Scl;
    else
        Sa_c(3) = Sal;
        Sb_c(3) = Sbl;
        Sc_c(3) = Sch;
    end

    
    t1 = min([ta,tb,tc]);
    t3 = max([ta,tb,tc]);
    if((ta ~= t1)&(ta ~= t3))
        t2 = ta;
    elseif((tb ~= t1)&(tb ~= t3))
        t2 = tb;
    else
        t2 = tc;
    end
    

    N = NLevel;
    if(FdeltVPN == 0)
    	FV = FV1(1);
    	FV_o = FV2(1);
    	Fi = Fis(1);
    elseif(FdeltVPN == 1)
    	FV = FV2(1);
    	FV_o = FV1(1);
    	Fi = Fis(1);
    elseif(FdeltVPN == 2)
    	FV = FV1(2);
    	FV_o = FV2(2);
    	Fi = Fis(2);
    elseif(FdeltVPN == 3)
    	FV = FV2(2);
    	FV_o = FV1(2);
    	Fi = Fis(2);	
    elseif(FdeltVPN == 4)
    	FV = FV1(3);
    	FV_o = FV2(3);
    	Fi = Fis(3);
    elseif(FdeltVPN == 5)
    	FV = FV2(3);
    	FV_o = FV1(3);
    	Fi = Fis(3);
    end	
    
    for i = 1:4,
     	sa_t(i) = RSSTable1(Sa_c(i)*192*N*N+Sb_c(i)*192*N+Sc_c(i)*192+FdeltVPN*32+FV*16+Fi*8+FV_o*4+1);
         sb_t(i) = RSSTable1(Sa_c(i)*192*N*N+Sb_c(i)*192*N+Sc_c(i)*192+FdeltVPN*32+FV*16+Fi*8+FV_o*4+2);
         sc_t(i) = RSSTable1(Sa_c(i)*192*N*N+Sb_c(i)*192*N+Sc_c(i)*192+FdeltVPN*32+FV*16+Fi*8+FV_o*4+3);
         Row =  RSSTable1(Sa_c(i)*192*N*N+Sb_c(i)*192*N+Sc_c(i)*192+FdeltVPN*32+FV*16+Fi*8+FV_o*4+4);

         Row_a(i) = RSSTable(sa_t(i)*16+FV1(1)*8+FV2(1)*4+Fis(1)*2+FdeltVP(1)+1);
         Row_b(i) = RSSTable(sb_t(i)*16+FV1(2)*8+FV2(2)*4+Fis(2)*2+FdeltVP(2)+1);
         Row_c(i) = RSSTable(sc_t(i)*16+FV1(3)*8+FV2(3)*4+Fis(3)*2+FdeltVP(3)+1);
     
         if((FdeltVPN == 0)||(FdeltVPN == 1))
             Row_a(i) = Row;
         elseif((FdeltVPN == 2)||(FdeltVPN == 3))
             Row_b(i) = Row;
         elseif((FdeltVPN == 4)||(FdeltVPN == 5))
             Row_c(i) = Row;
         end
     end         
    

    mVal = m;
    AlterIndex = 1;

    sys = [Row_a';Row_b';Row_c';t1;t2;t3;Tpos;AlterIndex;mVal];
    
% end mdlOutputs

