/*
 * sfuntmpl_basic.c: Basic 'C' template for a level 2 S-function.
 *
 *  -------------------------------------------------------------------------
 *  | See matlabroot/simulink/src/sfuntmpl_doc.c for a more detailed template |
 *  -------------------------------------------------------------------------
 *
 * Copyright 1990-2002 The MathWorks, Inc.
 * $Revision: 1.27 $
 */


/*
 * You must specify the S_FUNCTION_NAME as the name of your S-function
 * (i.e. replace sfuntmpl_basic with the name of your S-function).
 */

#define S_FUNCTION_NAME  PWMTime1
#define S_FUNCTION_LEVEL 2
#define D_pi             3.1415926

/*
 * Need to include simstruc.h for the definition of the SimStruct and
 * its associated macro definitions.
 */
#include "simstruc.h"




/* Error handling
 * --------------
 *
 * You should use the following technique to report errors encountered within
 * an S-function:
 *
 *       ssSetErrorStatus(S,"Error encountered due to ...");
 *       return;
 *
 * Note that the 2nd argument to ssSetErrorStatus must be persistent memory.
 * It cannot be a local variable. For example the following will cause
 * unpredictable errors:
 *
 *      mdlOutputs()
 *      {
 *         char msg[256];         {ILLEGAL: to fix use "static char msg[256];"}
 *         sprintf(msg,"Error due to %s", string);
 *         ssSetErrorStatus(S,msg);
 *         return;
 *      }
 *
 * See matlabroot/simulink/src/sfuntmpl_doc.c for more details.
 */

/*====================*
 * S-function methods *
 *====================*/

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    /* See sfuntmpl_doc.c for more details on the macros below */

    ssSetNumSFcnParams(S,2);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        /* Return if number of expected != number of actual parameters */
        return;
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 9)) return;
    ssSetInputPortWidth(S, 0, 1);
    ssSetInputPortWidth(S, 1, 4);
    ssSetInputPortWidth(S, 2, 4);
    ssSetInputPortWidth(S, 3, 4);
    ssSetInputPortWidth(S, 4, 1);
    ssSetInputPortWidth(S, 5, 1);
    ssSetInputPortWidth(S, 6, 1);
    ssSetInputPortWidth(S, 7, 1);
    ssSetInputPortWidth(S, 8, 1);

    ssSetInputPortRequiredContiguous(S, 0, true); /*direct input signal access*/
    ssSetInputPortRequiredContiguous(S, 1, true); /*direct input signal access*/
    ssSetInputPortRequiredContiguous(S, 2, true); /*direct input signal access*/
    ssSetInputPortRequiredContiguous(S, 3, true); /*direct input signal access*/
    ssSetInputPortRequiredContiguous(S, 4, true); /*direct input signal access*/
    ssSetInputPortRequiredContiguous(S, 5, true); /*direct input signal access*/
    ssSetInputPortRequiredContiguous(S, 6, true); /*direct input signal access*/
    ssSetInputPortRequiredContiguous(S, 7, true); /*direct input signal access*/
    ssSetInputPortRequiredContiguous(S, 8, true); /*direct input signal access*/    

    /*
     * Set direct feedthrough flag (1=yes, 0=no).
     * A port has direct feedthrough if the input is used in either
     * the mdlOutputs or mdlGetTimeOfNextVarHit functions.
     * See matlabroot/simulink/src/sfuntmpl_directfeed.txt.
     */
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    ssSetInputPortDirectFeedThrough(S, 4, 1);
    ssSetInputPortDirectFeedThrough(S, 5, 1);
    ssSetInputPortDirectFeedThrough(S, 6, 1);
    ssSetInputPortDirectFeedThrough(S, 7, 1);
    ssSetInputPortDirectFeedThrough(S, 8, 0);

    
    if (!ssSetNumOutputPorts(S, 3)) return;
    ssSetOutputPortWidth(S, 0, 3);
    ssSetOutputPortWidth(S, 1, 3);
    ssSetOutputPortWidth(S, 2, 3);
    
    ssSetNumSampleTimes(S,1);
    ssSetNumRWork(S, 1);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    ssSetOptions(S, 0);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *    This function is used to specify the sample time(s) for your
 *    S-function. You must register the same number of sample times as
 *    specified in ssSetNumSampleTimes.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, CONTINUOUS_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);

}



#define MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#if defined(MDL_INITIALIZE_CONDITIONS)
  /* Function: mdlInitializeConditions ========================================
   * Abstract:
   *    In this function, you should initialize the continuous and discrete
   *    states for your S-function block.  The initial states are placed
   *    in the state vector, ssGetContStates(S) or ssGetRealDiscStates(S).
   *    You can also perform any other initialization activities that your
   *    S-function may require. Note, this routine will be called at the
   *    start of simulation and if it is present in an enabled subsystem
   *    configured to reset states, it will be call when the enabled subsystem
   *    restarts execution to reset the states.
   */
  static void mdlInitializeConditions(SimStruct *S)
  {
  }
#endif /* MDL_INITIALIZE_CONDITIONS */



#define MDL_START  /* Change to #undef to remove function */
#if defined(MDL_START) 
  /* Function: mdlStart =======================================================
   * Abstract:
   *    This function is called once at start of model execution. If you
   *    have states that should be initialized once, this is the place
   *    to do it.
   */
  static void mdlStart(SimStruct *S)
  {
        
  }
#endif /*  MDL_START */



/* Function: mdlOutputs =======================================================
 * Abstract:
 *    In this function, you compute the outputs of your S-function
 *    block. Generally outputs are placed in the output vector, ssGetY(S).
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    const real_T *t = (const real_T*) ssGetInputPortSignal(S,0);
    const real_T *Row_a = (const real_T*) ssGetInputPortSignal(S,1);
    const real_T *Row_b = (const real_T*) ssGetInputPortSignal(S,2);
    const real_T *Row_c = (const real_T*) ssGetInputPortSignal(S,3);
    const real_T *time1 = (const real_T*) ssGetInputPortSignal(S,4);
    const real_T *time2 = (const real_T*) ssGetInputPortSignal(S,5);
    const real_T *time3 = (const real_T*) ssGetInputPortSignal(S,6);
    const real_T *Tpos = (const real_T*) ssGetInputPortSignal(S,7);
    const real_T *AlterIndex = (const real_T*) ssGetInputPortSignal(S,8);
    
    
    real_T       *T1 = ssGetOutputPortSignal(S,0);
    real_T       *T2 = ssGetOutputPortSignal(S,1);
    real_T       *T3 = ssGetOutputPortSignal(S,2);
    
    const real_T       *JustVal = (const real_T*)mxGetPr(ssGetSFcnParam(S,0));
    const real_T       *fsw = (const real_T*)mxGetPr(ssGetSFcnParam(S,1));
    
    const int        T3table[8] = {0,0,0,0,1,1,1,1};
    const int        T2table[8] = {0,0,1,1,0,0,1,1};
    const int        T1table[8] = {0,1,0,1,0,1,0,1};
       
    real_T      TPerPeriod;
    real_T      Tsw;
    
    Tsw = 1/fsw[0];
    
    TPerPeriod = t[0]-Tpos[0];      

   
    if(TPerPeriod >=0)
    {
        if(TPerPeriod >=time1[0])
        {
           if(TPerPeriod >=time2[0])
           {
              if(TPerPeriod >=time3[0])
              {
                T1[0] = T1table[(int)Row_a[3]];
                T2[0] = T2table[(int)Row_a[3]];
                T3[0] = T3table[(int)Row_a[3]];
        
                T1[1] = T1table[(int)Row_b[3]];
                T2[1] = T2table[(int)Row_b[3]];
                T3[1] = T3table[(int)Row_b[3]];
        
                T1[2] = T1table[(int)Row_c[3]];
                T2[2] = T2table[(int)Row_c[3]];
                T3[2] = T3table[(int)Row_c[3]];
              }
              else
              {
                T1[0] = T1table[(int)Row_a[2]];
                T2[0] = T2table[(int)Row_a[2]];
                T3[0] = T3table[(int)Row_a[2]];
        
                T1[1] = T1table[(int)Row_b[2]];
                T2[1] = T2table[(int)Row_b[2]];
                T3[1] = T3table[(int)Row_b[2]];
        
                T1[2] = T1table[(int)Row_c[2]];
                T2[2] = T2table[(int)Row_c[2]];
                T3[2] = T3table[(int)Row_c[2]];
              }
           }
           else
           {
                T1[0] = T1table[(int)Row_a[1]];
                T2[0] = T2table[(int)Row_a[1]];
                T3[0] = T3table[(int)Row_a[1]];
        
                T1[1] = T1table[(int)Row_b[1]];
                T2[1] = T2table[(int)Row_b[1]];
                T3[1] = T3table[(int)Row_b[1]];
        
                T1[2] = T1table[(int)Row_c[1]];
                T2[2] = T2table[(int)Row_c[1]];
                T3[2] = T3table[(int)Row_c[1]];
           }
        }
        else
        {
                T1[0] = T1table[(int)Row_a[0]];
                T2[0] = T2table[(int)Row_a[0]];
                T3[0] = T3table[(int)Row_a[0]];
        
                T1[1] = T1table[(int)Row_b[0]];
                T2[1] = T2table[(int)Row_b[0]];
                T3[1] = T3table[(int)Row_b[0]];
        
                T1[2] = T1table[(int)Row_c[0]];
                T2[2] = T2table[(int)Row_c[0]];
                T3[2] = T3table[(int)Row_c[0]];
        }
     }
  
}


#define MDL_UPDATE  /* Change to #undef to remove function */
#if defined(MDL_UPDATE)
  /* Function: mdlUpdate ======================================================
   * Abstract:
   *    This function is called once for every major integration time step.
   *    Discrete states are typically updated here, but this function is useful
   *    for performing any tasks that should only take place once per
   *    integration step.
   */
  static void mdlUpdate(SimStruct *S, int_T tid)
  {
  }
#endif /* MDL_UPDATE */



#define MDL_DERIVATIVES  /* Change to #undef to remove function */
#if defined(MDL_DERIVATIVES)
  /* Function: mdlDerivatives =================================================
   * Abstract:
   *    In this function, you compute the S-function block's derivatives.
   *    The derivatives are placed in the derivative vector, ssGetdX(S).
   */
  static void mdlDerivatives(SimStruct *S)
  {
  }
#endif /* MDL_DERIVATIVES */



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{
}


/*======================================================*
 * See sfuntmpl_doc.c for the optional S-function methods *
 *======================================================*/

/*=============================*
 * Required S-function trailer *
 *=============================*/

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
